import { CSSProperties, useEffect, useMemo, useState } from 'react'
// import Stories from 'react-insta-stories'
// import { Story } from 'react-insta-stories/dist/interfaces'
import { Swiper, SwiperSlide, useSwiper, useSwiperSlide } from 'swiper/react'
import { Virtual, Keyboard, Mousewheel, Zoom, Autoplay } from 'swiper/modules'

import { BottomNavBar, AnnouncementDetails, Filters } from '../components'
import { useFilters, useStoryDimensions, useStoryIndex } from '../hooks'
import { useAnnouncements } from '../hooks/api'
import { Announcement } from '../api/announcement/types'
import { categoryGraphics } from '../assets/category'
import { UseFetchReturn, gotError } from '../hooks/useFetch'

import 'swiper/css'
import 'swiper/css/virtual'
import 'swiper/css/zoom'
import 'swiper/css/keyboard'
import 'swiper/css/mousewheel'
import 'swiper/css/autoplay'

import puffSpinner from '../assets/puff.svg'
import { SetState } from '../utils/types'
import { Button } from 'react-bootstrap'
import { throttle } from '../utils'

const styles = {
    container: {
        display: 'flex',
        justifyContent: 'center',
        backgroundColor: 'rgb(17, 17, 17)',
        height: 'calc(100dvh - 56px)',
    } as CSSProperties,
    detailsContainer: {
        position: 'fixed',
        top: 0,
        zIndex: 1000,
        textAlign: 'initial',
    } as CSSProperties,
    cardContainer: {
        height: '100%',
        margin: '0 auto',
        position: 'relative',
    } as CSSProperties,
    cardMedia: {
        height: '100%',
    } as CSSProperties,
    cardDetails: {
        position: 'absolute',
        width: '100%',
        bottom: 0,
        textAlign: 'center',
        height: '56px',
        boxShadow: 'inset 0px -50px 36px -28px rgba(0, 0, 0, 0.35)',
    } as CSSProperties,
    progressBar: {
        height: '0.5dvh',
        width: 'calc(100% - 2dvh)',
        left: '1dvh',
        position: 'absolute',
        top: '1dvh',
        zIndex: 1000,
    } as CSSProperties,
}

type CardProps = {
    announcement: Announcement
    detailsShown: boolean
    percentage: number
    setDetailsShown: SetState<boolean>
}

const Card = ({
    announcement,
    detailsShown,
    percentage,
    setDetailsShown,
}: CardProps) => {
    const swiperSlide = useSwiperSlide()
    const swiper = useSwiper()

    useEffect(() => {
        if (detailsShown) swiper.autoplay.stop()
        else swiper.autoplay.start()
    }, [detailsShown])

    return (
        <>
            <AnnouncementDetails
                className="swiper-no-swiping"
                show={detailsShown && swiperSlide.isActive}
                announcement={announcement}
                close={() => setDetailsShown(false)}
                refresh={() => {}}
            />

            <div style={styles.cardContainer}>
                {swiperSlide.isActive && (
                    <div style={styles.progressBar}>
                        <div
                            style={{
                                background: 'white',
                                height: '100%',
                                width: (1 - percentage) * 100 + '%',
                                transition: 'width 0.1s linear',
                            }}
                        ></div>
                    </div>
                )}
                <div className="swiper-zoom-container">
                    {announcement.src?.endsWith('mp4') ||
                    announcement.src?.endsWith('webm') ? (
                        <video
                            style={styles.cardMedia}
                            src={announcement.src}
                            autoPlay={swiperSlide.isActive}
                        />
                    ) : (
                        <img
                            style={styles.cardMedia}
                            src={
                                announcement.src ||
                                categoryGraphics[announcement.category]
                            }
                            loading="lazy"
                        />
                    )}
                </div>
                <div style={styles.cardDetails}>
                    <Button
                        variant="success"
                        onClick={() => setDetailsShown(true)}
                    >
                        Подробности
                    </Button>
                </div>
            </div>
        </>
    )
}

function HomePage() {
    const { height, width } = useStoryDimensions(16 / 9)

    const [filterShown, setFilterShown] = useState(false)
    const [detailsShown, setDetailsShown] = useState(false)

    const [filter, setFilter] = useFilters()

    const announcements = useAnnouncements(filter)

    const index = useStoryIndex(announcements.data?.length)

    const [autoplayPercentage, setAutoplayPercentage] = useState(0)

    return (
        <>
            <Filters
                filter={filter}
                setFilter={index.withReset(setFilter)}
                filterShown={filterShown}
                setFilterShown={setFilterShown}
            />
            <div style={styles.container}>
                {!announcements.loading && !gotError(announcements) && (
                    <Swiper
                        autoplay={{ delay: 11000, stopOnLastSlide: true }}
                        direction="vertical"
                        keyboard
                        loop
                        modules={[
                            Autoplay,
                            Keyboard,
                            Mousewheel,
                            Virtual,
                            Zoom,
                        ]}
                        mousewheel
                        onAutoplayTimeLeft={throttle(
                            (_: unknown, __: unknown, percentage: number) =>
                                setAutoplayPercentage(percentage),
                            100
                        )}
                        noSwiping={true}
                        onActiveIndexChange={(s) => index.set(s.activeIndex)}
                        slidesPerView={1}
                        spaceBetween={0}
                        virtual
                        zoom={{}}
                        initialSlide={index.n}
                        onZoomChange={(s, scale) => {
                            if (scale == 1) {
                                s.enable()
                                s.autoplay.start()
                            } else {
                                s.disable()
                                s.autoplay.stop()
                            }
                        }}
                    >
                        {announcements.data.map((announcement, i) => (
                            <SwiperSlide
                                key={announcement.id}
                                virtualIndex={i}
                                zoom
                            >
                                <Card
                                    announcement={announcement}
                                    detailsShown={detailsShown}
                                    percentage={autoplayPercentage}
                                    setDetailsShown={setDetailsShown}
                                />
                            </SwiperSlide>
                        ))}
                    </Swiper>
                )}
            </div>
            <BottomNavBar toggleFilters={setFilterShown} width={width} />
        </>
    )
}

export default HomePage
